@@ -49,7 +49,7 @@ struct tkt_st {
 	Ticket *next;
 	IDnum id_a;
 	IDnum id_b;
-};
+} ATTRIBUTE_PACKED;
 
 //Global variables used throughout this procedure(internal use only !)
 static int MAXREADLENGTH = 100;
@@ -72,8 +72,8 @@ static int WORDLENGTH;
 static Graph *graph;
 static IDnum dbgCounter;
 
-static PassageMarker *fastPath;
-static PassageMarker *slowPath;
+static PassageMarkerI fastPath;
+static PassageMarkerI slowPath;
 
 static IDnum *eligibleStartingPoints;
 
@@ -89,7 +89,7 @@ static Ticket **todo;
 static Ticket *done;
 static boolean *progressStatus;
 
-static Coordinate *sequenceLengths;
+static IDnum *sequenceLengths;
 static Category *sequenceCategories;
 
 //End of global variables;
@@ -148,7 +148,7 @@ static boolean isPreviousToNode(Node * previous, Node * target)
 	Node *previousNode = NULL;
 	Time targetTime = getNodeTime(target);
 
-	//printf("Testing if %ld is previous to %ld\n", getNodeID(previous), getNodeID(target));
+	//velvetLog("Testing if %ld is previous to %ld\n", getNodeID(previous), getNodeID(target));
 
 	while (true) {
 		if (currentNode == previous)
@@ -177,7 +177,7 @@ static void concatenateCommonTodoLists(Node * nodeA, Node * nodeB)
 	IDnum indexA, indexB;
 	IDnum nodes = nodeCount(graph);
 
-	//printf("Merging todo list %ld into %ld\n", getNodeID(nodeB),
+	//velvetLog("Merging todo list %ld into %ld\n", getNodeID(nodeB),
 	//       getNodeID(nodeA));
 
 	if (*listB == NULL)
@@ -290,7 +290,7 @@ static void concatenateTodoListIntoActive(Node * nodeB)
 	IDnum activeID = getNodeID(activeNode);
 	IDnum indexB, indexA;
 
-	//printf("Merging todo list %ld into active node %ld\n",
+	//velvetLog("Merging todo list %ld into active node %ld\n",
 	//       getNodeID(nodeB), getNodeID(activeNode));
 
 	if (*listB == NULL)
@@ -479,7 +479,7 @@ static void determineEligibleStartingPoints()
 	IDnum counter = 0;
 	FibHeap *heap = newFibHeap();
 
-	puts("Determining eligible starting points");
+	velvetLog("Determining eligible starting points\n");
 
 	for (nodeIndex = 1; nodeIndex <= nodeCount(graph); nodeIndex++) {
 		node = getNodeInGraph(graph, nodeIndex);
@@ -509,7 +509,7 @@ static void determineEligibleStartingPoints()
 		eligibleStartingPoints[counter++] = getNodeID(node);
 
 	destroyHeap(heap);
-	puts("Done listing starting nodes");
+	velvetLog("Done listing starting nodes\n");
 }
 
 static Node *nextStartingPoint()
@@ -530,13 +530,13 @@ static Node *nextStartingPoint()
 }
 
 static boolean
-extractSequence(PassageMarker * path, TightString * sequence)
+extractSequence(PassageMarkerI path, TightString * sequence)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Coordinate seqLength = 0;
 	Coordinate writeIndex = 0;
 
-	//printf("Extracting sequence %ld ... ", pathLength);
+	//velvetLog("Extracting sequence %ld ... ", pathLength);
 
 	//Measure length
 	for (marker = getNextInSequence(path); !isTerminal(marker);
@@ -662,7 +662,7 @@ static void mapSlowOntoFast()
 			slowToFastMapping[--slowIndex] = fastIndex - 1;
 
 		else {
-			puts("Error");
+			velvetLog("Error\n");
 			fflush(stdout);
 			abort();
 		}
@@ -718,11 +718,11 @@ static void remapNodeArcsOntoNeighbour(Node * source, Node * target)
 }
 
 static void remapNodeMarkersOntoNeighbour(Node * source,
-					  PassageMarker * sourceMarker,
+					  PassageMarkerI sourceMarker,
 					  Node * target,
-					  PassageMarker * targetMarker)
+					  PassageMarkerI targetMarker)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Coordinate offset;
 	IDnum sourceLength, index;
 	ShortReadMarker *sourceArray, *shortMarker;
@@ -740,7 +740,7 @@ static void remapNodeMarkersOntoNeighbour(Node * source,
 	Coordinate realTargetLength = getNodeLength(target);
 	Coordinate realSourceLength = getNodeLength(source);
 
-	while (getMarker(source) != NULL) {
+	while (getMarker(source) != NULL_IDX) {
 		marker = getMarker(source);
 		extractPassageMarker(marker);
 		transposePassageMarker(marker, target);
@@ -883,12 +883,12 @@ static void remapBackOfNodeArcsOntoNeighbour(Node * source, Node * target)
 
 static Coordinate
 remapBackOfNodeMarkersOntoNeighbour(Node * source,
-				    PassageMarker * sourceMarker,
+				    PassageMarkerI sourceMarker,
 				    Node * target,
-				    PassageMarker * targetMarker,
+				    PassageMarkerI targetMarker,
 				    boolean slowToFast)
 {
-	PassageMarker *marker, *newMarker, *previousMarker, *nextMarker;
+	PassageMarkerI marker, newMarker, previousMarker, nextMarker;
 	Coordinate halfwayPoint, halfwayPointOffset, breakpoint,
 	    newStartOffset, newFinishOffset;
 	Category cat;
@@ -934,7 +934,7 @@ remapBackOfNodeMarkersOntoNeighbour(Node * source,
 	halfwayPointOffset = realSourceLength - halfwayPoint;
 
 	// Complete markers
-	for (marker = getMarker(source); marker != NULL;
+	for (marker = getMarker(source); marker != NULL_IDX;
 	     marker = nextMarker) {
 		nextMarker = getNextInNode(marker);
 
@@ -1251,8 +1251,8 @@ static void foldSymmetricalNode(Node * node)
 	Node *twinNode = getTwinNode(node);
 	Node *tmp, *destination;
 	Arc *arc;
-	PassageMarker *oldMarker = getMarker(node);
-	PassageMarker *currentMarker, *newMarker, *previousMarker;
+	PassageMarkerI oldMarker = getMarker(node);
+	PassageMarkerI currentMarker, newMarker, previousMarker;
 	Coordinate halfwayPoint;
 	IDnum totalMult;
 
@@ -1263,10 +1263,10 @@ static void foldSymmetricalNode(Node * node)
 		node = tmp;
 	}
 	// Destroy link to old markers 
-	setMarker(node, NULL);
+	setMarker(node, NULL_IDX);
 
 	// Reinsert markers properly
-	while (oldMarker != NULL) {
+	while (oldMarker != NULL_IDX) {
 		currentMarker = oldMarker;
 		oldMarker = getNextInNode(currentMarker);
 		previousMarker = getPreviousInSequence(currentMarker);
@@ -1379,22 +1379,22 @@ static void remapNodeTimesOntoNeighbour(Node * source, Node * target)
 }
 
 static void remapNodeTimesOntoForwardMiddlePath(Node * source,
-						PassageMarker * path)
+						PassageMarkerI path)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Node *target;
 	Time nodeTime = getNodeTime(source);
 	Node *previousNode = getNodePrevious(source);
 	Time targetTime;
 
-	//printf("Remapping times from %ld to %ld\n", getNodeID(previousNode), getNodeID(source));
+	//velvetLog("Remapping times from %ld to %ld\n", getNodeID(previousNode), getNodeID(source));
 
 	for (marker = path; getNode(marker) != source;
 	     marker = getNextInSequence(marker)) {
 		target = getNode(marker);
 		targetTime = getNodeTime(target);
 
-		//printf("Through %ld\n", getNodeID(target));
+		//velvetLog("Through %ld\n", getNodeID(target));
 
 		if (targetTime == -1
 		    || targetTime > nodeTime
@@ -1412,16 +1412,16 @@ static void remapNodeTimesOntoForwardMiddlePath(Node * source,
 }
 
 static void remapNodeTimesOntoTwinMiddlePath(Node * source,
-					     PassageMarker * path)
+					     PassageMarkerI path)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Node *target;
 	Node *previousNode = getTwinNode(source);
 	Time targetTime;
-	PassageMarker *limit = getTwinMarker(getPreviousInSequence(path));
+	PassageMarkerI limit = getTwinMarker(getPreviousInSequence(path));
 	Time nodeTime = getNodeTime(getNode(limit));
 
-	//printf("Remapping times from twins %ld to %ld\n", getNodeID(previousNode), getNodeID(getNode(limit)));
+	//velvetLog("Remapping times from twins %ld to %ld\n", getNodeID(previousNode), getNodeID(getNode(limit)));
 
 	// Revving up
 	marker = path;
@@ -1435,7 +1435,7 @@ static void remapNodeTimesOntoTwinMiddlePath(Node * source,
 		target = getNode(marker);
 		targetTime = getNodeTime(target);
 
-		//printf("Through %ld\n", getNodeID(target));
+		//velvetLog("Through %ld\n", getNodeID(target));
 
 		if (targetTime == -1
 		    || targetTime > nodeTime
@@ -1473,11 +1473,11 @@ remapNodeFibHeapReferencesOntoNode(Node * source, Node * target)
 }
 
 static void remapNodeOntoNeighbour(Node * source,
-				   PassageMarker * sourceMarker,
+				   PassageMarkerI sourceMarker,
 				   Node * target,
-				   PassageMarker * targetMarker)
+				   PassageMarkerI targetMarker)
 {
-	//printf("Remapping node %ld onto middle path %ld\n", getNodeID(source), getNodeID(target));
+	//velvetLog("Remapping node %ld onto middle path %ld\n", getNodeID(source), getNodeID(target));
 	remapNodeMarkersOntoNeighbour(source, sourceMarker, target,
 				      targetMarker);
 
@@ -1497,15 +1497,13 @@ static void remapNodeOntoNeighbour(Node * source,
 }
 
 static void remapBackOfNodeDescriptorOntoNeighbour(Node * source,
-						   PassageMarker *
-						   sourceMarker,
+						   PassageMarkerI sourceMarker,
 						   Node * target,
-						   PassageMarker *
-						   targetMarker,
+						   PassageMarkerI targetMarker,
 						   boolean slowToFast,
 						   Coordinate offset)
 {
-	//printf("Splitting node descriptor %ld // %ld\n", getNodeLength(source), offset);
+	//velvetLog("Splitting node descriptor %ld // %ld\n", getNodeLength(source), offset);
 
 	if (slowToFast)
 		splitNodeDescriptor(source, NULL, offset);
@@ -1559,12 +1557,12 @@ static void remapBackOfNodeTimesOntoNeighbour(Node * source, Node * target)
 }
 
 static void
-remapBackOfNodeOntoNeighbour(Node * source, PassageMarker * sourceMarker,
-			     Node * target, PassageMarker * targetMarker,
+remapBackOfNodeOntoNeighbour(Node * source, PassageMarkerI sourceMarker,
+			     Node * target, PassageMarkerI targetMarker,
 			     boolean slowToFast)
 {
 	Coordinate offset;
-	//printf("Remapping node %ld onto middle path\n", getNodeID(node));
+	//velvetLog("Remapping node %ld onto middle path\n", getNodeID(node));
 
 	offset =
 	    remapBackOfNodeMarkersOntoNeighbour(source, sourceMarker,
@@ -1583,12 +1581,10 @@ remapBackOfNodeOntoNeighbour(Node * source, PassageMarker * sourceMarker,
 		startingNode = getTwinNode(target);
 }
 
-static void remapEmptyPathArcsOntoMiddlePathSimple(PassageMarker *
-						   emptyPath,
-						   PassageMarker *
-						   targetPath)
+static void remapEmptyPathArcsOntoMiddlePathSimple(PassageMarkerI emptyPath,
+						   PassageMarkerI targetPath)
 {
-	PassageMarker *pathMarker;
+	PassageMarkerI pathMarker;
 	Node *start = getNode(getPreviousInSequence(emptyPath));
 	Node *finish = getNode(emptyPath);
 	Node *previousNode = start;
@@ -1608,15 +1604,13 @@ static void remapEmptyPathArcsOntoMiddlePathSimple(PassageMarker *
 	destroyArc(originalArc, graph);
 }
 
-static void remapEmptyPathMarkersOntoMiddlePathSimple(PassageMarker *
-						      emptyPath,
-						      PassageMarker *
-						      targetPath)
+static void remapEmptyPathMarkersOntoMiddlePathSimple(PassageMarkerI emptyPath,
+						      PassageMarkerI targetPath)
 {
-	PassageMarker *marker, *newMarker, *previousMarker, *pathMarker;
+	PassageMarkerI marker, newMarker, previousMarker, pathMarker;
 	Node *start = getNode(getPreviousInSequence(emptyPath));
 	Node *finish = getNode(emptyPath);
-	PassageMarker *oldMarker = getMarker(finish);
+	PassageMarkerI oldMarker = getMarker(finish);
 	Coordinate markerStart;
 	IDnum intersectionLength, twinIntersectionLength;
 	ShortReadMarker *intersectionReads =
@@ -1625,14 +1619,14 @@ static void remapEmptyPathMarkersOntoMiddlePathSimple(PassageMarker *
 	    commonNodeReads(getTwinNode(start), getTwinNode(finish), graph,
 			    &twinIntersectionLength);
 
-	//printf("SIMPLE %ld\t%ld\t%i\t%i\n", markerCount(finish),
+	//velvetLog("SIMPLE %ld\t%ld\t%i\t%i\n", markerCount(finish),
 	//       getNodeID(finish), arcCount(finish),
 	//       arcCount(getTwinNode(finish)));
 
 	// Destroy link to old nodes
-	setMarker(finish, NULL);
+	setMarker(finish, NULL_IDX);
 
-	while (oldMarker != NULL) {
+	while (oldMarker != NULL_IDX) {
 		marker = oldMarker;
 		oldMarker = getNextInNode(marker);
 		newMarker = getPreviousInSequence(marker);
@@ -1690,17 +1684,17 @@ static void remapEmptyPathMarkersOntoMiddlePathSimple(PassageMarker *
 	free(twinIntersectionReads);
 }
 
-static boolean markerFollowsPath(PassageMarker * marker,
-				 PassageMarker * start,
-				 PassageMarker * finish, Node * stopNode)
+static boolean markerFollowsPath(PassageMarkerI marker,
+				 PassageMarkerI start,
+				 PassageMarkerI finish, Node * stopNode)
 {
-	PassageMarker *current, *path;
+	PassageMarkerI current, path;
 
 	path = start;
 	current = marker;
 
 	while (true) {
-		if (current == NULL || path == finish || path == NULL)
+		if (current == NULL_IDX || path == finish || path == NULL_IDX)
 			return true;
 
 		if (getNode(current) != getNode(path))
@@ -1711,16 +1705,16 @@ static boolean markerFollowsPath(PassageMarker * marker,
 	}
 }
 
-static PassageMarkerList *getAnchors(PassageMarker * marker, Node * nodeA,
+static PassageMarkerList *getAnchors(PassageMarkerI marker, Node * nodeA,
 				     Node * nodeB)
 {
-	PassageMarker *current, *next;
+	PassageMarkerI current, next;
 	Node *twinA = getTwinNode(nodeA);
 	Node *twinB = getTwinNode(nodeB);
 	PassageMarkerList *result = NULL;
 
 	current = marker;
-	while (current != NULL) {
+	while (current != NULL_IDX) {
 		next = getNextInSequence(current);
 		if (getNode(current) == nodeA && getNode(next) == nodeB) {
 			result = newPassageMarkerList(next, result);
@@ -1747,12 +1741,10 @@ static void destroyPassageMarkerList(PassageMarkerList ** list)
 	}
 }
 
-static void remapEmptyPathMarkersOntoMiddlePathDevious(PassageMarker *
-						       emptyPath,
-						       PassageMarker *
-						       targetPath)
+static void remapEmptyPathMarkersOntoMiddlePathDevious(PassageMarkerI emptyPath,
+						       PassageMarkerI targetPath)
 {
-	PassageMarker *marker, *newMarker, *previousMarker, *pathMarker;
+	PassageMarkerI marker, newMarker, previousMarker, pathMarker;
 	Node *start = getNode(getPreviousInSequence(emptyPath));
 	Node *finish = getNode(emptyPath);
 	PassageMarkerList *anchors = getAnchors(targetPath, start, finish);
@@ -1760,11 +1752,11 @@ static void remapEmptyPathMarkersOntoMiddlePathDevious(PassageMarker *
 	boolean untouchable = false;
 	Coordinate markerStart;
 
-	printf("DEVIOUS %d\t%d\t%i\t%i\n", markerCount(finish),
-	       getNodeID(finish), arcCount(finish),
-	       arcCount(getTwinNode(finish)));
+	velvetLog("DEVIOUS %li\t%li\t%li\t%li\n", (long) markerCount(finish),
+	       (long) getNodeID(finish), (long) arcCount(finish),
+	       (long) arcCount(getTwinNode(finish)));
 
-	for (marker = getMarker(finish); marker != NULL;
+	for (marker = getMarker(finish); marker != NULL_IDX;
 	     marker = getNextInNode(marker)) {
 		newMarker = getPreviousInSequence(marker);
 
@@ -1813,15 +1805,15 @@ static void remapEmptyPathMarkersOntoMiddlePathDevious(PassageMarker *
 	destroyPassageMarkerList(&anchors);
 }
 
-static boolean markerLeadsToArc(PassageMarker * marker, Node * nodeA,
+static boolean markerLeadsToArc(PassageMarkerI marker, Node * nodeA,
 				Node * nodeB)
 {
-	PassageMarker *current, *next;
+	PassageMarkerI current, next;
 	Node *twinA = getTwinNode(nodeA);
 	Node *twinB = getTwinNode(nodeB);
 
 	current = marker;
-	while (current != NULL) {
+	while (current != NULL_IDX) {
 		next = getNextInSequence(current);
 		if (getNode(current) == nodeA && getNode(next) == nodeB)
 			return true;
@@ -1834,8 +1826,8 @@ static boolean markerLeadsToArc(PassageMarker * marker, Node * nodeA,
 }
 
 static void
-remapEmptyPathOntoMiddlePath(PassageMarker * emptyPath,
-			     PassageMarker * targetPath)
+remapEmptyPathOntoMiddlePath(PassageMarkerI emptyPath,
+			     PassageMarkerI targetPath)
 {
 	Node *start = getNode(getPreviousInSequence(emptyPath));
 	Node *finish = getNode(emptyPath);
@@ -1859,9 +1851,9 @@ remapEmptyPathOntoMiddlePath(PassageMarker * emptyPath,
 		remapNodeTimesOntoTwinMiddlePath(finish, targetPath);
 }
 
-static void reduceSlowNodes(PassageMarker * slowMarker, Node * finish)
+static void reduceSlowNodes(PassageMarkerI slowMarker, Node * finish)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
 	for (marker = slowMarker; getNode(marker) != finish;
 	     marker = getNextInSequence(marker)) {
@@ -1871,9 +1863,9 @@ static void reduceSlowNodes(PassageMarker * slowMarker, Node * finish)
 
 static void destroyPaths()
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
-	while (slowPath != NULL) {
+	while (slowPath != NULL_IDX) {
 		marker = slowPath;
 		getNodeTime(getNode(marker));
 		getNodeTime(getTwinNode(getNode(marker)));
@@ -1882,7 +1874,7 @@ static void destroyPaths()
 		destroyPassageMarker(marker);
 	}
 
-	while (fastPath != NULL) {
+	while (fastPath != NULL_IDX) {
 		marker = fastPath;
 		getNodeTime(getNode(marker));
 		getNodeTime(getTwinNode(getNode(marker)));
@@ -1893,7 +1885,7 @@ static void destroyPaths()
 
 static Coordinate mapDistancesOntoPaths()
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Coordinate totalDistance = 0;
 
 	marker = slowPath;
@@ -1916,11 +1908,11 @@ static Coordinate mapDistancesOntoPaths()
 	return totalDistance;
 }
 
-static boolean markerLeadsToNode(PassageMarker * marker, Node * node)
+static boolean markerLeadsToNode(PassageMarkerI marker, Node * node)
 {
-	PassageMarker *currentMarker;
+	PassageMarkerI currentMarker;
 
-	for (currentMarker = marker; currentMarker != NULL;
+	for (currentMarker = marker; currentMarker != NULL_IDX;
 	     currentMarker = getNextInSequence(currentMarker))
 		if (getNode(currentMarker) == node)
 			return true;
@@ -1998,13 +1990,13 @@ static void transferNodeData(Node * source, Node * target)
 static void concatenateNodesAndVaccinate(Node * nodeA, Node * nodeB,
 					 Graph * graph)
 {
-	PassageMarker *marker, *tmpMarker;
+	PassageMarkerI marker, tmpMarker;
 	Node *twinA = getTwinNode(nodeA);
 	Node *twinB = getTwinNode(nodeB);
 	Arc *arc;
 	Category cat;
 
-	//printf("Concatenating nodes %ld and %ld\n", getNodeID(nodeA), getNodeID(nodeB));
+	//velvetLog("Concatenating nodes %ld and %ld\n", getNodeID(nodeA), getNodeID(nodeB));
 	// Arc management:
 	// Freeing useless arcs
 	while (getArc(nodeA) != NULL)
@@ -2021,14 +2013,14 @@ static void concatenateNodesAndVaccinate(Node * nodeA, Node * nodeB,
 	}
 
 	// Passage marker management in node A:
-	for (marker = getMarker(nodeA); marker != NULL;
+	for (marker = getMarker(nodeA); marker != NULL_IDX;
 	     marker = getNextInNode(marker))
 		if (isTerminal(marker))
 			incrementFinishOffset(marker,
 					      getNodeLength(nodeB));
 
 	// Swapping new born passageMarkers from B to A
-	for (marker = getMarker(nodeB); marker != NULL; marker = tmpMarker) {
+	for (marker = getMarker(nodeB); marker != NULL_IDX; marker = tmpMarker) {
 		tmpMarker = getNextInNode(marker);
 
 		if (isInitial(marker)) {
@@ -2089,12 +2081,12 @@ static void simplifyNode(Node * node)
 
 }
 
-static void concatenatePathNodes(PassageMarker * pathStart)
+static void concatenatePathNodes(PassageMarkerI pathStart)
 {
-	PassageMarker *pathMarker;
+	PassageMarkerI pathMarker;
 
-	//puts("Removing null loops");
-	for (pathMarker = pathStart; pathMarker != NULL;
+	//velvetLog("Removing null loops\n");
+	for (pathMarker = pathStart; pathMarker != NULL_IDX;
 	     pathMarker = getNextInSequence(pathMarker)) {
 		simplifyNode(getNode(pathMarker));
 	}
@@ -2105,19 +2097,19 @@ static void concatenatePathNodes(PassageMarker * pathStart)
 
 static void cleanUpRedundancy()
 {
-	PassageMarker *slowMarker = getNextInSequence(slowPath);
-	PassageMarker *fastMarker = getNextInSequence(fastPath);
+	PassageMarkerI slowMarker = getNextInSequence(slowPath);
+	PassageMarkerI fastMarker = getNextInSequence(fastPath);
 	Coordinate slowLength, fastLength;
 	Coordinate fastConstraint = 0;
 	Coordinate slowConstraint = 0;
 	Coordinate finalLength;
 	Node *slowNode, *fastNode;
 
-	//puts("Correcting new redundancy");
+	//velvetLog("Correcting new redundancy\n");
 	mapSlowOntoFast();
 	finalLength = mapDistancesOntoPaths();
 
-	while (slowMarker != NULL && fastMarker != NULL) {
+	while (slowMarker != NULL_IDX && fastMarker != NULL_IDX) {
 		if (isTerminal(slowMarker))
 			slowLength = finalLength;
 		else {
@@ -2138,16 +2130,16 @@ static void cleanUpRedundancy()
 		if (slowNode == fastNode) {
 			if (fastLength > slowLength)
 				slowConstraint = fastLength;
-			else if (fastLength < slowLength);
-			fastConstraint = slowLength;
+			else if (fastLength < slowLength)
+				fastConstraint = slowLength;
 
 			slowMarker = getNextInSequence(slowMarker);
 			fastMarker = getNextInSequence(fastMarker);
 		} else if (slowNode == getTwinNode(fastNode)) {
 			if (fastLength > slowLength)
 				slowConstraint = fastLength;
-			else if (fastLength < slowLength);
-			fastConstraint = slowLength;
+			else if (fastLength < slowLength)
+				fastConstraint = slowLength;
 
 			slowMarker = getNextInSequence(slowMarker);
 			fastMarker = getNextInSequence(fastMarker);
@@ -2183,34 +2175,34 @@ static void cleanUpRedundancy()
 		fflush(stdout);
 	}
 
-	//puts("Done with path");
+	//velvetLog("Done with path\n");
 
 	while (!isInitial(slowPath))
 		slowPath = getPreviousInSequence(slowPath);
 	while (!isInitial(fastPath))
 		fastPath = getPreviousInSequence(fastPath);
 
-	//puts("Concatenation");
+	//velvetLog("Concatenation\n");
 
 	// Freeing up memory  
-	if (slowMarker != NULL)
+	if (slowMarker != NULL_IDX)
 		concatenatePathNodes(slowPath);
 	else
 		concatenatePathNodes(fastPath);
 
-	//puts("Vaccinatting");
+	//velvetLog("Vaccinatting\n");
 
 	destroyPaths();
 
 	// Cleaning up silly structures
 	//vaccinatePath(&returnValue);
 
-	//puts("Clean up done");
+	//velvetLog("Clean up done\n");
 	//fflush(stdout);
 }
 
-static boolean pathContainsReference(PassageMarker * path) {
-	PassageMarker *marker, *marker2;
+static boolean pathContainsReference(PassageMarkerI path) {
+	PassageMarkerI marker, marker2;
 
 	for (marker = getNextInSequence(path); !isTerminal(marker);
 	     marker = getNextInSequence(marker))
@@ -2227,7 +2219,7 @@ static void comparePaths(Node * destination, Node * origin)
 	IDnum slowLength, fastLength;
 	Node *fastNode, *slowNode;
 	IDnum i;
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
 	//Measure lengths
 	slowLength = fastLength = 0;
@@ -2314,7 +2306,7 @@ static void comparePaths(Node * destination, Node * origin)
 		cleanUpRedundancy();
 		return;
 	}
-	//puts("\tFinished comparing paths, changes made");
+	//velvetLog("\tFinished comparing paths, changes made\n");
 	destroyPaths();
 }
 
@@ -2342,7 +2334,7 @@ static void tourBusArc(Node * origin, Arc * arc, Time originTime)
 		return;
 	} else if (destinationTime > totalTime) {
 		if (dheapNodes[nodeIndex] == NULL) {
-			//puts("Already expanded though");
+			//velvetLog("Already expanded though\n");
 			return;
 		}
 
@@ -2373,7 +2365,7 @@ static void initializeTodoLists()
 	Ticket *currentTicket, *tmp;
 	Node *destination;
 
-	puts("Initializing todo lists");
+	velvetLog("Initializing todo lists\n");
 
 	for (index = -nodes; index <= nodes; index++) {
 		node = getNodeInGraph(graph, index);
@@ -2411,7 +2403,7 @@ static void initializeTodoLists()
 		}
 	}
 
-	puts("Done with initilization");
+	velvetLog("Done with initilization\n");
 }
 
 static void tourBusNode(Node * node)
@@ -2423,7 +2415,7 @@ static void tourBusNode(Node * node)
 
 	dbgCounter++;
 	if (dbgCounter % 1000 == 0) {
-		printf("%d nodes visited\n", dbgCounter);
+		velvetLog("%li nodes visited\n", (long) dbgCounter);
 		fflush(stdout);
 	}
 
@@ -2510,9 +2502,9 @@ void clipTips(Graph * graph)
 	Node *current, *twin;
 	boolean modified = true;
 	int Wordlength = getWordLength(graph);
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
-	puts("Clipping short tips off graph");
+	velvetLog("Clipping short tips off graph\n");
 
 	while (modified) {
 		modified = false;
@@ -2555,7 +2547,7 @@ void clipTips(Graph * graph)
 	}
 
 	concatenateGraph(graph);
-	printf("%d nodes left\n", nodeCount(graph));
+	velvetLog("%li nodes left\n", (long) nodeCount(graph));
 }
 
 void clipTipsHard(Graph * graph)
@@ -2564,9 +2556,9 @@ void clipTipsHard(Graph * graph)
 	Node *current, *twin;
 	boolean modified = true;
 	int Wordlength = getWordLength(graph);
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
-	puts("Clipping short tips off graph, drastic");
+	velvetLog("Clipping short tips off graph, drastic\n");
 
 	while (modified) {
 		modified = false;
@@ -2608,7 +2600,7 @@ void clipTipsHard(Graph * graph)
 	}
 
 	concatenateGraph(graph);
-	printf("%d nodes left\n", nodeCount(graph));
+	velvetLog("%li nodes left\n", (long) nodeCount(graph));
 }
 
 static void tourBus(Node * startingPoint)
@@ -2616,7 +2608,7 @@ static void tourBus(Node * startingPoint)
 	Node *currentNode = startingPoint;
 	IDnum nodeID = getNodeID(startingPoint) + nodeCount(graph);
 
-	//printf("Tour bus from node %ld...\n", (long) getNodeID(startingPoint));
+	//velvetLog("Tour bus from node %ld...\n", (long) getNodeID(startingPoint));
 
 	times[nodeID] = 0;
 	previous[nodeID] = currentNode;
@@ -2629,7 +2621,7 @@ static void tourBus(Node * startingPoint)
 	}
 }
 
-void correctGraph(Graph * argGraph, Coordinate * argSequenceLengths, Category * argSequenceCategories)
+void correctGraph(Graph * argGraph, IDnum * argSequenceLengths, Category * argSequenceCategories)
 {
 	IDnum nodes;
 	IDnum index;
@@ -2642,7 +2634,7 @@ void correctGraph(Graph * argGraph, Coordinate * argSequenceLengths, Category *
 	dbgCounter = 0;
 	// Done with global params
 
-	printf("Correcting graph with cutoff %f\n", MAXDIVERGENCE);
+	velvetLog("Correcting graph with cutoff %f\n", MAXDIVERGENCE);
 
 	//clipTips(graph);
 	nodes = nodeCount(graph);
@@ -2679,7 +2671,7 @@ void correctGraph(Graph * argGraph, Coordinate * argSequenceLengths, Category *
 	activateArcLookupTable(graph);
 
 	while ((startingNode = nextStartingPoint()) != NULL) {
-		//puts("Going through the cycle...");
+		//velvetLog("Going through the cycle...\n");
 		tourBus(startingNode);
 		updateNodeStatus(startingNode);
 	}
@@ -2717,8 +2709,11 @@ void correctGraph(Graph * argGraph, Coordinate * argSequenceLengths, Category *
 void setMaxReadLength(int value)
 {
 	if (value < 0) {
-		printf("Negative branch length %i!\n", value);
-		puts("Exiting...");
+		velvetLog("Negative branch length %i!\n", value);
+		velvetLog("Exiting...\n");
+#ifdef DEBUG 
+		abort();
+#endif 
 		exit(1);
 	}
 	MAXREADLENGTH = value;
@@ -2728,8 +2723,11 @@ void setMaxReadLength(int value)
 void setMaxGaps(int value)
 {
 	if (value < 0) {
-		printf("Negative max gap count %i!\n", value);
-		puts("Exiting...");
+		velvetLog("Negative max gap count %i!\n", value);
+		velvetLog("Exiting...\n");
+#ifdef DEBUG 
+		abort();
+#endif 
 		exit(1);
 	}
 	MAXGAPS = value;
@@ -2738,9 +2736,12 @@ void setMaxGaps(int value)
 void setMaxDivergence(double value)
 {
 	if (value < 0 || value > 1) {
-		printf("Divergence rate %lf out of bounds [0,1]!\n",
+		velvetLog("Divergence rate %lf out of bounds [0,1]!\n",
 		       value);
-		puts("Exiting...");
+		velvetLog("Exiting...\n");
+#ifdef DEBUG 
+		abort();
+#endif 
 		exit(1);
 	}
 	MAXDIVERGENCE = value;
