@@ -46,7 +46,7 @@ typedef struct smallNodeList_st SmallNodeList;
 struct smallNodeList_st {
 	Node *node;
 	SmallNodeList *next;
-};
+} ATTRIBUTE_PACKED;
 
 static RecycleBin *smallNodeListMemory = NULL;
 static SmallNodeList *nodePile = NULL;
@@ -93,12 +93,12 @@ static void unlockMemorizedNodes()
 typedef struct referenceMapping_st ReferenceMapping;
 
 struct referenceMapping_st {
-	Coordinate referenceStart;
-	Coordinate nodeStart;
-	Coordinate length;
+	IDnum referenceStart;
+	IDnum nodeStart;
+	IDnum length;
 	IDnum referenceID;
 	IDnum nodeID;
-};
+} ATTRIBUTE_PACKED;
 
 static IDnum countMappings(char * preGraphFilename) {
 	FILE *file = fopen(preGraphFilename, "r");
@@ -248,9 +248,9 @@ typedef struct nodeMask_st NodeMask;
 
 struct nodeMask_st {
 	IDnum nodeID;
-	Coordinate start;
-	Coordinate finish;
-};
+	IDnum start;
+	IDnum finish;
+} ATTRIBUTE_PACKED;
 
 static int compareNodeMasks(const void * ptrA, const void * ptrB) {
 	NodeMask * A = (NodeMask *) ptrA;
@@ -325,7 +325,7 @@ static KmerOccurenceTable *referenceGraphKmers(char *preGraphFilename,
 		exitErrorf(EXIT_FAILURE, true, "Could not open %s", preGraphFilename);
 
 	// Count kmers
-	printf("Scanning pre-graph file %s for k-mers\n",
+	velvetLog("Scanning pre-graph file %s for k-mers\n",
 	       preGraphFilename);
 
 	// First  line
@@ -348,7 +348,7 @@ static KmerOccurenceTable *referenceGraphKmers(char *preGraphFilename,
 			break;
 	}
 
-	printf("%li kmers found\n", (long) kmerCount);
+	velvetLog("%li kmers found\n", (long) kmerCount);
 
 	for(nodeMaskIndex = 0; nodeMaskIndex < nodeMaskCount; nodeMaskIndex++) {
 		kmerCount -= nodeMasks[nodeMaskIndex].finish -
@@ -507,8 +507,22 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 
 	// Neglect any read which will not be short paired
 	if ((!readTracking && category % 2 == 0)
-	    || category / 2 >= CATEGORIES)
+	    || category / 2 >= CATEGORIES) {
+		if (*rdmapFile) {
+			if (!fgets(line, MAXLINE, *rdmapFile)) {
+				fclose(*rdmapFile);
+				*rdmapFile = NULL;
+			}
+			while (line[0] != 'R') {
+				if (!fgets(line, MAXLINE, *rdmapFile)) {
+					fclose(*rdmapFile);
+					*rdmapFile = NULL;
+					break;
+				}
+			}
+		}
 		return;
+	}
 
 	// Get initial annotation
 	if (*rdmapFile) {
@@ -534,14 +548,24 @@ static void ghostThreadSequenceThroughGraph(TightString * tString,
 	}
 	
 	// Neglect any string shorter than WORDLENGTH :
-	if (getLength(tString) < wordLength)
+	if (getLength(tString) < wordLength) {
+		if (*rdmapFile) {
+			while (line[0] != 'R') {
+				if (!fgets(line, MAXLINE, *rdmapFile)) {
+					fclose(*rdmapFile);
+					*rdmapFile = NULL;
+					break;
+				}
+			}
+		}
 		return;
+	}
 
 	// Verify that all short reads are reasonnably short
 	if (getLength(tString) > USHRT_MAX) {
-		printf("Short read of length %lli, longer than limit %i\n",
+		velvetLog("Short read of length %lli, longer than limit %i\n",
 		       (long long) getLength(tString), SHRT_MAX);
-		puts("You should better declare this sequence as long, because it genuinely is!");
+		velvetLog("You should better declare this sequence as long, because it genuinely is!\n");
 		exit(1);
 	}
 	// Allocate memory for the read pairs
@@ -705,8 +729,8 @@ static void threadSequenceThroughGraph(TightString * tString,
 	KmerOccurence *kmerOccurence;
 	int wordLength = getWordLength(graph);
 
-	PassageMarker *marker = NULL;
-	PassageMarker *previousMarker = NULL;
+	PassageMarkerI marker = NULL_IDX;
+	PassageMarkerI previousMarker = NULL_IDX;
 	Node *node;
 	Node *previousNode = NULL;
 	Coordinate coord = 0;
@@ -755,8 +779,18 @@ static void threadSequenceThroughGraph(TightString * tString,
 	}
 
 	// Neglect any string shorter than WORDLENGTH :
-	if (getLength(tString) < wordLength)
+	if (getLength(tString) < wordLength) {
+		if (*rdmapFile) {
+			while (line[0] != 'R') {
+				if (!fgets(line, MAXLINE, *rdmapFile)) {
+					fclose(*rdmapFile);
+					*rdmapFile = NULL;
+					break;
+				}
+			}
+		}
 		return;
+	}
 
 	// Fill in the initial word : 
 	for (readNucleotideIndex = 0;
@@ -1004,11 +1038,10 @@ static void fillUpGraph(ReadSet * reads,
 	boolean second_in_pair = false;
 	
 	if (referenceMappings) {
-		file = fopen(roadmapFilename, "r");
-		for (readIndex = 0; readIndex < refCount + 1; readIndex++)
-			while(fgets(line, MAXLINE, file))
-				if (line[0] == 'R')
-					break;
+	    file = fopen(roadmapFilename, "r");
+	    while(fgets(line, MAXLINE, file))
+		    if (line[0] == 'R')
+			    break;
 	}
 
 	resetNodeStatus(graph);
@@ -1016,8 +1049,7 @@ static void fillUpGraph(ReadSet * reads,
 	for (readIndex = 0; readIndex < reads->readCount; readIndex++) {
 		category = reads->categories[readIndex];
 	
-		ghostThreadSequenceThroughGraph(reads->
-						tSequences[readIndex],
+		ghostThreadSequenceThroughGraph(getTightStringInArray(reads->tSequences, readIndex),
 						kmerTable,
 						graph, readIndex + 1,
 						category, 
@@ -1035,10 +1067,10 @@ static void fillUpGraph(ReadSet * reads,
 	createNodeReadStartArrays(graph);
 
 	if (referenceMappings) {
-		file = fopen(roadmapFilename, "r");
-		while(fgets(line, MAXLINE, file))
-			if (line[0] == 'R')
-				break;
+	    file = fopen(roadmapFilename, "r");
+	    while(fgets(line, MAXLINE, file))
+		    if (line[0] == 'R')
+			    break;
 	}
 
 	second_in_pair = false;
@@ -1046,10 +1078,10 @@ static void fillUpGraph(ReadSet * reads,
 		category = reads->categories[readIndex];
 	
 		if (readIndex % 100000 == 0)
-			printf("Threading through reads %d / %d\n",
-			       readIndex, reads->readCount);
+			velvetLog("Threading through reads %li / %li\n",
+			       (long) readIndex, (long) reads->readCount);
 
-		threadSequenceThroughGraph(reads->tSequences[readIndex],
+		threadSequenceThroughGraph(getTightStringInArray(reads->tSequences, readIndex),
 					   kmerTable,
 					   graph, readIndex + 1, category,
 					   readTracking, double_strand,
@@ -1080,6 +1112,9 @@ Graph *importPreGraph(char *preGraphFilename, ReadSet * reads, char * roadmapFil
 	Coordinate referenceMappingCount = 0;
 	IDnum referenceCount = 0;
 
+	if (nodeCount(graph) == 0)
+		return graph;
+
 	// If necessary compile reference -> node
 	ReferenceMapping * referenceMappings = computeReferenceMappings(preGraphFilename, reads, &referenceMappingCount, &referenceCount); 
 	// Node -> reference maps
