@@ -33,33 +33,45 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 static void printUsage()
 {
 	puts("Usage:");
-	puts("./velveth directory hash_length {[-file_format][-read_type] filename} [options]");
+	puts("./velveth directory hash_length {[-file_format][-read_type] filename1 [filename2 ...]} {...} [options]");
 	puts("");
-	puts("\tdirectory\t\t: directory name for output files");
-	printf("\thash_length\t\t: odd integer (if even, it will be decremented) <= %i (if above, will be reduced)\n", MAXKMERLENGTH);
-	puts("\tfilename\t\t: path to sequence file or - for standard input");	
+	puts("\tdirectory\t: directory name for output files");
+	printf("\thash_length\t: odd integer (if even, it will be decremented) <= %i (if above, will be reduced)\n", MAXKMERLENGTH);
+	puts("\tfilename\t: path to sequence file or - for standard input");	
 	puts("");
 	puts("File format options:");
-	puts("\t-fasta");
-	puts("\t-fastq");
-	puts("\t-fasta.gz");
-	puts("\t-fastq.gz");
-	puts("\t-sam");
-	puts("\t-bam");
-	puts("\t-eland");
-	puts("\t-gerald");
+	puts("\t-fasta\t-fastq\t-raw\t-fasta.gz\t-fastq.gz\t-raw.gz\t-sam\t-bam");
 	puts("");
 	puts("Read type options:");
-	puts("\t-short");
-	puts("\t-shortPaired");
-	puts("\t-short2");
-	puts("\t-shortPaired2");
-	puts("\t-long");
-	puts("\t-longPaired");
+	puts("\t-short\t-shortPaired");
+#if CATEGORIES <= 5
+	Category cat; 
+	for (cat = 2; cat <= CATEGORIES; cat++)
+	    printf("\t-short%i\t-shortPaired%i\n", cat, cat);
+#else
+	puts("\t...");
+	printf("\t-short%i\t-shortPaired%i\n", CATEGORIES - 1, CATEGORIES - 1);
+	printf("\t-short%i\t-shortPaired%i\n", CATEGORIES, CATEGORIES);
+#endif
+	puts("\t-long\t-longPaired");
 	puts("");
 	puts("Options:");
 	puts("\t-strand_specific\t: for strand specific transcriptome sequencing data (default: off)");
 	puts("");
+	puts("Synopsis:");
+	puts("");
+	puts("- Short single end reads:");
+	puts("\tvelveth Assem 29 -short -fastq s_1_sequence.txt");
+	puts("");
+	puts("- Paired-end short reads (remember to interleave paired reads):");
+	puts("\tvelveth Assem 31 -shortPaired -fasta interleaved.fna");
+	puts("");
+	puts("- Two channels and some long reads:");
+	puts("\tvelveth Assem 43 -short -fastq unmapped.fna -longPaired -fasta SangerReads.fasta");
+	puts("");
+	puts("- Three channels:");
+	puts("\tvelveth Assem 35 -shortPaired -fasta pe_lib1.fasta -shortPaired2 pe_lib2.fasta -short3 se_lib1.fa");
+	puts("");
 	puts("Output:");
 	puts("\tdirectory/Roadmaps");
 	puts("\tdirectory/Sequences");
@@ -78,16 +90,16 @@ int main(int argc, char **argv)
 	setProgramName("velveth");
 
 	if (argc < 4) {
-		puts("velveth - simple hashing program");
+		printf("velveth - simple hashing program\n");
 		printf("Version %i.%i.%2.2i\n", VERSION_NUMBER,
 		       RELEASE_NUMBER, UPDATE_NUMBER);
-		puts("\nCopyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)");
-		puts("This is free software; see the source for copying conditions.  There is NO");
-		puts("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
-		puts("Compilation settings:");
+		printf("\nCopyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)\n");
+		printf("This is free software; see the source for copying conditions.  There is NO\n");
+		printf("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
+		printf("Compilation settings:\n");
 		printf("CATEGORIES = %i\n", CATEGORIES);
 		printf("MAXKMERLENGTH = %i\n", MAXKMERLENGTH);
-		puts("");
+		printf("\n");
 		printUsage();
 		return 0;
 	}
@@ -100,18 +112,18 @@ int main(int argc, char **argv)
 	hashLength = atoi(argv[2]);
 
 	if (hashLength > MAXKMERLENGTH) {
-		printf
+		velvetLog
 		    ("Velvet can't handle k-mers as long as %i! We'll stick to %i if you don't mind.\n",
 		     hashLength, MAXKMERLENGTH);
 		hashLength = MAXKMERLENGTH;
 	} else if (hashLength <= 0) {
-		printf("Invalid hash length: %s\n", argv[2]);
+		velvetLog("Invalid hash length: %s\n", argv[2]);
 		printUsage();
 		return 0;
 	} 
 
 	if (hashLength % 2 == 0) {
-		printf
+		velvetLog
 		    ("Velvet can't work with even length k-mers, such as %i. We'll use %i instead, if you don't mind.\n",
 		     hashLength, hashLength - 1);
 		hashLength--;
@@ -146,7 +158,7 @@ int main(int argc, char **argv)
 	splayTable = newSplayTable(hashLength, double_strand);
 
 	allSequences = importReadSet(seqFilename);
-	printf("%i sequences in total.\n", allSequences->readCount);
+	velvetLog("%li sequences in total.\n", (long) allSequences->readCount);
 
 	strcpy(filename, directory);
 	strcat(filename, "/Roadmaps");
