@@ -40,11 +40,13 @@ static void printUsage()
 	puts("\t-amos_file <yes|no>\t\t: export assembly to AMOS file (default: no export)");
 	puts("\t-exp_cov <floating point|auto>\t: expected coverage of unique regions or allow the system to infer it");
 	puts("\t\t(default: no long or paired-end read resolution)");
+	puts("\t-long_cov_cutoff <floating-point>: removal of nodes with low long-read coverage AFTER tour bus");
+	puts("\t\t(default: no removal)");
 	puts("");
 	puts("Advanced options:");
 	puts("\t-ins_length2 <integer>\t\t: expected distance between two paired-end reads in the second short-read dataset (default: no read pairing)");
 	puts("\t-ins_length_long <integer>\t: expected distance between two long paired-end reads (default: no read pairing)");
-	puts("\t-ins_length*_sd <integer>\t: est. standard deviation of respective dataset (default: 10\% of corresponding length)");
+	puts("\t-ins_length*_sd <integer>\t: est. standard deviation of respective dataset (default: 10% of corresponding length)");
 	puts("\t\t[replace '*' by nothing, '2' or '_long' as necessary]");
 	puts("\t-scaffolding <yes|no>\t\t: scaffolding of contigs used paired end information (default: on)");
 	puts("\t-max_branch_length <integer>\t: maximum length in base pair of bubble (default: 100)");
@@ -55,6 +57,9 @@ static void printUsage()
 	puts("\t-long_mult_cutoff <int>\t\t: minimum number of long reads required to merge contigs (default: 2)");
 	puts("\t-unused_reads <yes|no>\t\t: export unused reads in UnusedReads.fa file (default: no)");
 	puts("\t-alignments <yes|no>\t\t: export a summary of contig alignment to the reference sequences (default: no)");
+	puts("\t-exportFiltered <yes|no>\t: export the long nodes which were eliminated by the coverage filters (default: no)");
+	puts("\t-clean <yes|no>\t\t: remove all the intermediary files which are useless for recalculation (default : no)");
+	puts("\t-very_clean <yes|no>\t\t: remove all the intermediary files (no recalculation possible) (default: no)");
 	puts("");
 	puts("Output:");
 	puts("\tdirectory/contigs.fa\t\t: fasta file of contigs longer than twice hash length");
@@ -70,8 +75,9 @@ int main(int argc, char **argv)
 	PreGraph *preGraph;
 	Graph *graph;
 	char *directory, *graphFilename, *preGraphFilename, *seqFilename,
-	    *roadmapFilename;
+	    *roadmapFilename, *lowCovContigsFilename, *highCovContigsFilename;
 	double coverageCutoff = -1;
+	double longCoverageCutoff = -1;
 	double maxCoverageCutoff = -1;
 	double expectedCoverage = -1;
 	int longMultCutoff = -1;
@@ -93,12 +99,14 @@ int main(int argc, char **argv)
 	int arg_index, arg_int;
 	double arg_double;
 	char *arg;
-	Coordinate *sequenceLengths = NULL;
+	IDnum *sequenceLengths = NULL;
 	Category cat;
 	boolean scaffolding = true;
 	int pebbleRounds = 1;
 	long long longlong_var;
 	short int short_var;
+	boolean exportFilteredNodes = false;
+	int clean = 0;
 
 	setProgramName("velvetg");
 
@@ -112,9 +120,9 @@ int main(int argc, char **argv)
 		puts("velvetg - de Bruijn graph construction, error removal and repeat resolution");
 		printf("Version %i.%i.%2.2i\n", VERSION_NUMBER,
 		       RELEASE_NUMBER, UPDATE_NUMBER);
-		puts("\nCopyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)");
+		puts("Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)");
 		puts("This is free software; see the source for copying conditions.  There is NO");
-		puts("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
+		puts("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.");
 		puts("Compilation settings:");
 		printf("CATEGORIES = %i\n", CATEGORIES);
 		printf("MAXKMERLENGTH = %i\n", MAXKMERLENGTH);
@@ -135,12 +143,17 @@ int main(int argc, char **argv)
 	    mallocOrExit(strlen(directory) + 100, char);
 	roadmapFilename = mallocOrExit(strlen(directory) + 100, char);
 	seqFilename = mallocOrExit(strlen(directory) + 100, char);
+	lowCovContigsFilename = mallocOrExit(strlen(directory) + 100, char);
+	highCovContigsFilename = mallocOrExit(strlen(directory) + 100, char);
 	// Argument parsing
 	for (arg_index = 2; arg_index < argc; arg_index++) {
 		arg = argv[arg_index++];
 		if (arg_index >= argc) {
-			puts("Unusual number of arguments!");
+			velvetLog("Unusual number of arguments!\n");
 			printUsage();
+#ifdef DEBUG 
+			abort();
+#endif 
 			exit(1);
 		}
 
@@ -150,6 +163,8 @@ int main(int argc, char **argv)
 			} else {
 				sscanf(argv[arg_index], "%lf", &coverageCutoff);
 			}
+		} else if (strcmp(arg, "-long_cov_cutoff") == 0) {
+			sscanf(argv[arg_index], "%lf", &longCoverageCutoff);
 		} else if (strcmp(arg, "-exp_cov") == 0) {
 			if (strcmp(argv[arg_index], "auto") == 0) {
 				estimateCoverage = true;
@@ -163,16 +178,22 @@ int main(int argc, char **argv)
 			sscanf(argv[arg_index], "%lli", &longlong_var);
 			insertLength[0] = (Coordinate) longlong_var;
 			if (insertLength[0] < 0) {
-				printf("Invalid insert length: %lli\n",
+				velvetLog("Invalid insert length: %lli\n",
 				       (long long) insertLength[0]);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 		} else if (strcmp(arg, "-ins_length_sd") == 0) {
 			sscanf(argv[arg_index], "%lli", &longlong_var);
 			std_dev[0] = (Coordinate) longlong_var;
 			if (std_dev[0] < 0) {
-				printf("Invalid std deviation: %lli\n",
+				velvetLog("Invalid std deviation: %lli\n",
 				       (long long) std_dev[0]);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 		} else if (strcmp(arg, "-ins_length_long") == 0) {
@@ -186,28 +207,40 @@ int main(int argc, char **argv)
 			sscanf(arg, "-ins_length%hi", &short_var);
 			cat = (Category) short_var;
 			if (cat < 1 || cat > CATEGORIES) {
-				printf("Unknown option: %s\n", arg);
+				velvetLog("Unknown option: %s\n", arg);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 			sscanf(argv[arg_index], "%lli", &longlong_var);
 			insertLength[cat - 1] = (Coordinate) longlong_var;
 			if (insertLength[cat - 1] < 0) {
-				printf("Invalid insert length: %lli\n",
+				velvetLog("Invalid insert length: %lli\n",
 				       (long long) insertLength[cat - 1]);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 		} else if (strncmp(arg, "-ins_length", 11) == 0) {
 			sscanf(arg, "-ins_length%hi_sd", &short_var);
 			cat = (Category) short_var;
 			if (cat < 1 || cat > CATEGORIES) {
-				printf("Unknown option: %s\n", arg);
+				velvetLog("Unknown option: %s\n", arg);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 			sscanf(argv[arg_index], "%lli", &longlong_var);
 			std_dev[cat - 1] = (Coordinate) longlong_var;
 			if (std_dev[cat - 1] < 0) {
-				printf("Invalid std deviation: %lli\n",
+				velvetLog("Invalid std deviation: %lli\n",
 				       (long long) std_dev[cat - 1]);
+#ifdef DEBUG 
+				abort();
+#endif 
 				exit(1);
 			}
 		} else if (strcmp(arg, "-read_trkg") == 0) {
@@ -216,6 +249,9 @@ int main(int argc, char **argv)
 		} else if (strcmp(arg, "-scaffolding") == 0) {
 			scaffolding =
 			    (strcmp(argv[arg_index], "yes") == 0);
+		} else if (strcmp(arg, "-exportFiltered") == 0) {
+			exportFilteredNodes =
+			    (strcmp(argv[arg_index], "yes") == 0);
 		} else if (strcmp(arg, "-amos_file") == 0) {
 			exportAssembly =
 			    (strcmp(argv[arg_index], "yes") == 0);
@@ -228,7 +264,7 @@ int main(int argc, char **argv)
 		} else if (strcmp(arg, "-accel_bits") == 0) {
 			sscanf(argv[arg_index], "%hi", &accelerationBits);
 			if (accelerationBits < 0) {
-				printf
+				velvetLog
 				    ("Illegal acceleration parameter: %s\n",
 				     argv[arg_index]);
 				printUsage();
@@ -254,6 +290,12 @@ int main(int argc, char **argv)
 		} else if (strcmp(arg, "-long_mult_cutoff") == 0) {
 			sscanf(argv[arg_index], "%i", &longMultCutoff);
 			setMultiplicityCutoff(longMultCutoff);
+		} else if (strcmp(arg, "-clean") == 0) {
+			if (strcmp(argv[arg_index], "yes") == 0)
+				clean = 1;
+		} else if (strcmp(arg, "-very_clean") == 0) {
+			if (strcmp(argv[arg_index], "yes") == 0)
+				clean = 2;
 		} else if (strcmp(arg, "-unused_reads") == 0) {
 			unusedReads =
 			    (strcmp(argv[arg_index], "yes") == 0);
@@ -263,7 +305,7 @@ int main(int argc, char **argv)
 			printUsage();
 			return 0;	
 		} else {
-			printf("Unknown option: %s;\n", arg);
+			velvetLog("Unknown option: %s;\n", arg);
 			printUsage();
 			return 1;
 		}
@@ -289,6 +331,12 @@ int main(int argc, char **argv)
 		strcat(graphFilename, "/Graph2");
 	}
 
+	strcpy(lowCovContigsFilename, directory);
+	strcat(lowCovContigsFilename, "/lowCoverageContigs.fa");
+
+	strcpy(highCovContigsFilename, directory);
+	strcat(highCovContigsFilename, "/highCoverageContigs.fa");
+
 	// Graph uploading or creation
 	if ((file = fopen(graphFilename, "r")) != NULL) {
 		fclose(file);
@@ -324,7 +372,10 @@ int main(int argc, char **argv)
 		correctGraph(graph, sequenceLengths, sequences->categories);
 		exportGraph(graphFilename, graph, sequences->tSequences);
 	} else {
-		puts("No Roadmap file to build upon! Please run velveth (see manual)");
+		velvetLog("No Roadmap file to build upon! Please run velveth (see manual)\n");
+#ifdef DEBUG 
+		abort();
+#endif 
 		exit(1);
 	}
 
@@ -357,9 +408,9 @@ int main(int argc, char **argv)
 	}
 
 	if (coverageCutoff < 0) {
-		puts("WARNING: NO COVERAGE CUTOFF PROVIDED");
-		puts("Velvet will probably leave behind many detectable errors");
-		puts("See manual for instructions on how to set the coverage cutoff parameter");
+		velvetLog("WARNING: NO COVERAGE CUTOFF PROVIDED\n");
+		velvetLog("Velvet will probably leave behind many detectable errors\n");
+		velvetLog("See manual for instructions on how to set the coverage cutoff parameter\n");
 	}
 
 	if (sequences == NULL) {
@@ -367,11 +418,28 @@ int main(int argc, char **argv)
 		convertSequences(sequences);
 	}
 
+	if (minContigLength < 2 * getWordLength(graph))
+		minContigKmerLength = getWordLength(graph);
+	else
+		minContigKmerLength = minContigLength - getWordLength(graph) + 1;		
+
 	dubious =
 	    removeLowCoverageNodesAndDenounceDubiousReads(graph,
-							  coverageCutoff,							
-							  sequences);
-	removeHighCoverageNodes(graph, maxCoverageCutoff);
+							  coverageCutoff,
+							  sequences,
+							  exportFilteredNodes,
+							  minContigKmerLength,
+							  lowCovContigsFilename);
+
+	removeLowLongCoverageNodesAndDenounceDubiousReads(graph,
+							  longCoverageCutoff,
+							  sequences,
+							  dubious,
+							  exportFilteredNodes,
+							  minContigKmerLength,
+							  lowCovContigsFilename);
+
+	removeHighCoverageNodes(graph, maxCoverageCutoff, exportFilteredNodes, minContigKmerLength, highCovContigsFilename);
 	clipTipsHard(graph);
 
 	if (expectedCoverage > 0) {
@@ -394,20 +462,15 @@ int main(int argc, char **argv)
 		for ( ;pebbleRounds > 0; pebbleRounds--)
 			exploitShortReadPairs(graph, sequences, dubious, scaffolding);
 	} else {
-		puts("WARNING: NO EXPECTED COVERAGE PROVIDED");
-		puts("Velvet will be unable to resolve any repeats");
-		puts("See manual for instructions on how to set the expected coverage parameter");
+		velvetLog("WARNING: NO EXPECTED COVERAGE PROVIDED\n");
+		velvetLog("Velvet will be unable to resolve any repeats\n");
+		velvetLog("See manual for instructions on how to set the expected coverage parameter\n");
 	}
 
 	free(dubious);
 
 	concatenateGraph(graph);
 
-	if (minContigLength < 2 * getWordLength(graph))
-		minContigKmerLength = getWordLength(graph);
-	else
-		minContigKmerLength = minContigLength - getWordLength(graph) + 1;		
-
 	strcpy(graphFilename, directory);
 	strcat(graphFilename, "/contigs.fa");
 	exportLongNodeSequences(graphFilename, graph, minContigKmerLength); 
@@ -423,9 +486,11 @@ int main(int argc, char **argv)
 	strcat(graphFilename, "/stats.txt");
 	displayGeneralStatistics(graph, graphFilename, sequences);
 
-	strcpy(graphFilename, directory);
-	strcat(graphFilename, "/LastGraph");
-	exportGraph(graphFilename, graph, sequences->tSequences);
+	if (clean == 0) {
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/LastGraph");
+		exportGraph(graphFilename, graph, sequences->tSequences);
+	}
 
 	if (exportAssembly) {
 		strcpy(graphFilename, directory);
@@ -437,17 +502,43 @@ int main(int argc, char **argv)
 		exportUnusedReads(graph, sequences, minContigKmerLength, directory);
 
 	if (estimateCoverage) 
-		printf("Estimated Coverage = %f\n", expectedCoverage);
+		velvetLog("Estimated Coverage = %f\n", expectedCoverage);
 	if (estimateCutoff) 
-		printf("Estimated Coverage cutoff = %f\n", coverageCutoff);
+		velvetLog("Estimated Coverage cutoff = %f\n", coverageCutoff);
 
 	logFinalStats(graph, minContigKmerLength, directory);
 
+	if (clean > 0) {
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/Roadmaps");
+		remove(graphFilename);	
+
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/Sequences");
+		remove(graphFilename);	
+
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/LastGraph");
+		remove(graphFilename);	
+	} 
+
+	if (clean > 1) {
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/Graph2.txt");
+		remove(graphFilename);	
+
+		strcpy(graphFilename, directory);
+		strcat(graphFilename, "/Graph.txt");
+		remove(graphFilename);	
+	}
+
 	destroyGraph(graph);
 	free(graphFilename);
 	free(preGraphFilename);
 	free(seqFilename);
 	free(roadmapFilename);
+	free(lowCovContigsFilename);
+	free(highCovContigsFilename);
 	destroyReadSet(sequences);
 	return 0;
 }
