@@ -47,13 +47,13 @@ struct connection_st {
 	double variance;
 	IDnum direct_count;
 	IDnum paired_count;
-};
+}  ATTRIBUTE_PACKED;
 
 struct readOccurence_st {
-	Coordinate position;
-	Coordinate offset;
+	IDnum position;
+	IDnum offset;
 	IDnum nodeID;
-};
+}  ATTRIBUTE_PACKED;
 
 // Global params
 static IDnum UNRELIABLE_CONNECTION_CUTOFF = 5;
@@ -173,7 +173,7 @@ void destroyConnection(Connection * connect, IDnum nodeID)
 {
 	Connection *previous, *next;
 
-	//printf("Destroying connection from %li to %li\n", nodeID, getNodeID(connect->destination));
+	//velvetLog("Destroying connection from %li to %li\n", nodeID, getNodeID(connect->destination));
 
 	if (connect == NULL)
 		return;
@@ -227,13 +227,13 @@ static IDnum *computeReadToNodeCounts()
 	IDnum maxNodeIndex = 2 * nodeCount(graph) + 1;
 	IDnum maxReadIndex = sequenceCount(graph) + 1;
 	IDnum *readNodeCounts = callocOrExit(maxReadIndex, IDnum);
-	boolean *readMarker = callocOrExit(maxReadIndex, boolean);
+	boolean *readMarker = callocOrExit(maxReadIndex, boolean); /* SF TODO this could be a bit field */
 	ShortReadMarker *nodeArray, *shortMarker;
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Node *node;
 	IDnum nodeReadCount;
 
-	puts("Computing read to node mapping array sizes");
+	velvetLog("Computing read to node mapping array sizes\n");
 
 	for (nodeIndex = 0; nodeIndex < maxNodeIndex; nodeIndex++) {
 		node = getNodeInGraph(graph, nodeIndex - nodeCount(graph));
@@ -252,7 +252,7 @@ static IDnum *computeReadToNodeCounts()
 		}
 
 		// Long reads
-		for (marker = getMarker(node); marker != NULL;
+		for (marker = getMarker(node); marker != NULL_IDX;
 		     marker = getNextInNode(marker)) {
 			readIndex = getPassageMarkerSequenceID(marker);
 			if (readIndex < 0)
@@ -266,7 +266,7 @@ static IDnum *computeReadToNodeCounts()
 		}
 
 		// Clean up marker array
-		for (marker = getMarker(node); marker != NULL;
+		for (marker = getMarker(node); marker != NULL_IDX;
 		     marker = getNextInNode(marker)) {
 			readIndex = getPassageMarkerSequenceID(marker);
 			if (readIndex > 0)
@@ -300,7 +300,8 @@ static ReadOccurence **allocateReadToNodeTables(IDnum * readNodeCounts)
 static void computePartialReadToNodeMapping(IDnum nodeID,
 					    ReadOccurence ** readNodes,
 					    IDnum * readNodeCounts,
-					    boolean * readMarker)
+					    boolean * readMarker,
+					    ReadSet * reads)
 {
 	ShortReadMarker *shortMarker;
 	IDnum index, readIndex;
@@ -308,7 +309,7 @@ static void computePartialReadToNodeMapping(IDnum nodeID,
 	Node *node = getNodeInGraph(graph, nodeID);
 	ShortReadMarker *nodeArray = getNodeReads(node, graph);
 	IDnum nodeReadCount = getNodeReadCount(node, graph);
-	PassageMarker *marker;
+	PassageMarkerI marker;
 
 	for (index = 0; index < nodeReadCount; index++) {
 		shortMarker = getShortReadMarkerAtIndex(nodeArray, index);
@@ -323,10 +324,10 @@ static void computePartialReadToNodeMapping(IDnum nodeID,
 		readNodeCounts[readIndex]++;
 	}
 
-	for (marker = getMarker(node); marker != NULL;
+	for (marker = getMarker(node); marker != NULL_IDX;
 	     marker = getNextInNode(marker)) {
 		readIndex = getPassageMarkerSequenceID(marker);
-		if (readIndex < 0)
+		if (readIndex <= 0 || reads->categories[readIndex - 1] == REFERENCE)
 			continue;
 
 		if (!readMarker[readIndex]) {
@@ -348,7 +349,7 @@ static void computePartialReadToNodeMapping(IDnum nodeID,
 		}
 	}
 
-	for (marker = getMarker(node); marker != NULL;
+	for (marker = getMarker(node); marker != NULL_IDX;
 	     marker = getNextInNode(marker)) {
 		readIndex = getPassageMarkerSequenceID(marker);
 		if (readIndex > 0)
@@ -356,22 +357,23 @@ static void computePartialReadToNodeMapping(IDnum nodeID,
 	}
 }
 
-static ReadOccurence **computeReadToNodeMappings(IDnum * readNodeCounts)
+static ReadOccurence **computeReadToNodeMappings(IDnum * readNodeCounts, ReadSet * reads)
 {
 	IDnum nodeID;
 	IDnum nodes = nodeCount(graph);
 	ReadOccurence **readNodes =
 	    allocateReadToNodeTables(readNodeCounts);
 	boolean *readMarker =
-	    callocOrExit(sequenceCount(graph) + 1, boolean);
+	    callocOrExit(sequenceCount(graph) + 1, boolean); /* SF TODO This could be a bit field */
 
-	puts("Computing read to node mappings");
+	velvetLog("Computing read to node mappings\n");
 
 	for (nodeID = -nodes; nodeID <= nodes; nodeID++)
 		if (nodeID != 0 && getNodeInGraph(graph, nodeID))
 			computePartialReadToNodeMapping(nodeID, readNodes,
 							readNodeCounts,
-							readMarker);
+							readMarker,
+							reads);
 
 	free(readMarker);
 	return readNodes;
@@ -382,7 +384,7 @@ static boolean * countCoOccurences(IDnum * coOccurencesCount, ReadOccurence ** r
 	IDnum readNodeCount;
 	IDnum readOccurenceIndex, readPairOccurenceIndex;
 	ReadOccurence * readOccurence, *readPairOccurence;
-	boolean * interestingReads = callocOrExit(sequenceCount(graph), boolean);
+	boolean * interestingReads = callocOrExit(sequenceCount(graph), boolean); /* SF TODO This could be a bit field */
 	Category libID;
 
 	for (libID = 0; libID < CATEGORIES + 1; libID++)
@@ -531,7 +533,7 @@ static void estimateLibraryInsertLength(Coordinate * coOccurences, IDnum coOccur
 	if (variance == 0)
 		variance = 1;
 
-	printf("Paired-end library %i has length: %lli, sample standard deviation: %lli\n", libID + 1, (long long) median, (long long) sqrt(variance));
+	velvetLog("Paired-end library %i has length: %lli, sample standard deviation: %lli\n", libID + 1, (long long) median, (long long) sqrt(variance));
 	setInsertLengths(graph, libID, median, sqrt(variance));
 	estimated[libID] = true;
 }
@@ -548,16 +550,16 @@ static void estimateLibraryInsertLengths(Coordinate ** coOccurences, IDnum * coO
 }
 
 static void estimateMissingInsertLengths(ReadOccurence ** readNodes, IDnum * readNodeCounts, IDnum * readPairs, Category * cats) {
-	Coordinate * coOccurences[CATEGORIES + 1];
+	Coordinate * coOccurences[CATEGORIES + 1]; /* SF TODO This could probably be done with IDnum */
 	IDnum coOccurencesCounts[CATEGORIES + 1]; 
 	Category libID;
 
-	puts("Estimating library insert lengths...");
+	velvetLog("Estimating library insert lengths...\n");
 
 	boolean * interestingReads = countCoOccurences(coOccurencesCounts, readNodes, readNodeCounts, readPairs, cats);
 
 	for (libID = 0; libID < CATEGORIES + 1; libID++)
-		coOccurences[libID] = callocOrExit(coOccurencesCounts[libID], Coordinate);
+		coOccurences[libID] = callocOrExit(coOccurencesCounts[libID], Coordinate); /* SF TODO This could probably be done with IDnum */
 
 	measureCoOccurences(coOccurences, interestingReads, readNodes, readNodeCounts, readPairs, cats);
 	estimateLibraryInsertLengths(coOccurences, coOccurencesCounts);
@@ -567,7 +569,7 @@ static void estimateMissingInsertLengths(ReadOccurence ** readNodes, IDnum * rea
 	
 	free(interestingReads);
 
-	puts("Done");
+	velvetLog("Done\n");
 }
 
 static Connection *findConnection(IDnum nodeID, IDnum node2ID)
@@ -823,7 +825,7 @@ static void projectFromShortRead(Node * node,
 				 IDnum * readPairs, Category * cats,
 				 ReadOccurence ** readNodes,
 				 IDnum * readNodeCounts,
-				 Coordinate * lengths)
+				 IDnum * lengths)
 {
 	IDnum index;
 	IDnum readIndex = getShortReadMarkerID(shortMarker);
@@ -863,11 +865,11 @@ static void projectFromShortRead(Node * node,
 
 }
 
-static void projectFromLongRead(Node * node, PassageMarker * marker,
+static void projectFromLongRead(Node * node, PassageMarkerI marker,
 				IDnum * readPairs, Category * cats,
 				ReadOccurence ** readNodes,
 				IDnum * readNodeCounts,
-				Coordinate * lengths)
+				IDnum * lengths)
 {
 	IDnum index;
 	IDnum readIndex = getPassageMarkerSequenceID(marker);
@@ -912,11 +914,11 @@ static void projectFromNode(IDnum nodeID,
 			    ReadOccurence ** readNodes,
 			    IDnum * readNodeCounts,
 			    IDnum * readPairs, Category * cats,
-			    boolean * dubious, Coordinate * lengths)
+			    boolean * dubious, IDnum * lengths)
 {
 	IDnum index;
 	ShortReadMarker *nodeArray, *shortMarker;
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Node *node;
 	IDnum nodeReadCount;
 
@@ -935,7 +937,7 @@ static void projectFromNode(IDnum nodeID,
 				     readNodes, readNodeCounts, lengths);
 	}
 
-	for (marker = getMarker(node); marker != NULL;
+	for (marker = getMarker(node); marker != NULL_IDX;
 	     marker = getNextInNode(marker)) {
 		if (getPassageMarkerSequenceID(marker) > 0)
 			projectFromLongRead(node, marker, readPairs, cats,
@@ -949,17 +951,17 @@ static Connection **computeNodeToNodeMappings(ReadOccurence ** readNodes,
 					      IDnum * readPairs,
 					      Category * cats,
 					      boolean * dubious,
-					      Coordinate * lengths)
+					      IDnum * lengths)
 {
 	IDnum nodeID;
 	IDnum nodes = nodeCount(graph);
 	scaffold = callocOrExit(2 * nodes + 1, Connection *);
 
-	puts("Computing direct node to node mappings");
+	velvetLog("Computing direct node to node mappings\n");
 
 	for (nodeID = -nodes; nodeID <= nodes; nodeID++) {
 		if (nodeID % 10000 == 0)
-			printf("Scaffolding node %d\n", nodeID);
+			velvetLog("Scaffolding node %li\n", (long) nodeID);
 
 		projectFromNode(nodeID, readNodes, readNodeCounts,
 				readPairs, cats, dubious, lengths);
@@ -1016,7 +1018,7 @@ void printConnections(ReadSet * reads)
 	IDnum **counts = countShortReads(graph, reads);
 	IDnum nodes = nodeCount(graph);
 
-	puts("CONNECT IDA IDB dcount pcount dist lengthA lengthB var countA countB coordA coordB real exp distance test");
+	velvetLog("CONNECT IDA IDB dcount pcount dist lengthA lengthB var countA countB coordA coordB real exp distance test\n");
 
 	for (index = 0; index < maxNodeIndex; index++) {
 		node = getNodeInGraph(graph, index - nodeCount(graph));
@@ -1024,7 +1026,7 @@ void printConnections(ReadSet * reads)
 		     connect = next) {
 			next = connect->next;
 			if (getUniqueness(connect->destination)) {
-				printf
+				velvetLog
 				    ("CONNECT %ld %ld %ld %ld %lld %lld %lld %f %ld %ld",
 				     (long) index - nodeCount(graph),
 				     (long) getNodeID(connect->destination),
@@ -1040,7 +1042,7 @@ void printConnections(ReadSet * reads)
 				if (markerCount(node) == 1
 				    && markerCount(connect->destination) ==
 				    1)
-					printf(" %lld %lld %lld",
+					velvetLog(" %lld %lld %lld",
 					       (long long) getPassageMarkerFinish
 					       (getMarker(node)),
 					       (long long) getPassageMarkerFinish
@@ -1052,14 +1054,14 @@ void printConnections(ReadSet * reads)
 					       (getMarker
 						(connect->destination))));
 				else
-					printf(" ? ?");
-				printf(" %ld", (long) expectedNumberOfConnections(index-nodeCount(graph), connect, counts, 0));
-				printf(" %lld", (long long) (getConnectionDistance(connect) - (getNodeLength(node) + getNodeLength(connect->destination))/2));
+					velvetLog(" ? ?");
+				velvetLog(" %ld", (long) expectedNumberOfConnections(index-nodeCount(graph), connect, counts, 0));
+				velvetLog(" %lld", (long long) (getConnectionDistance(connect) - (getNodeLength(node) + getNodeLength(connect->destination))/2));
 				if (testConnection
 				    (index - nodes, connect, counts))
-					puts(" OK");
+					velvetLog(" OK\n");
 				else
-					puts(" NG");
+					velvetLog(" NG\n");
 			}
 		}
 	}
@@ -1096,7 +1098,7 @@ void buildScaffold(Graph * argGraph, ReadSet * reads, boolean * dubious) {
 	Category *cats;
 	IDnum *readNodeCounts;
 	ReadOccurence **readNodes;
-	Coordinate *lengths =
+	IDnum *lengths =
 	    getSequenceLengths(reads, getWordLength(argGraph));
 	IDnum index;
 
@@ -1106,7 +1108,7 @@ void buildScaffold(Graph * argGraph, ReadSet * reads, boolean * dubious) {
 
 	// Prepare primary scaffold
 	readNodeCounts = computeReadToNodeCounts();
-	readNodes = computeReadToNodeMappings(readNodeCounts);
+	readNodes = computeReadToNodeMappings(readNodeCounts, reads);
 
 	estimateMissingInsertLengths(readNodes, readNodeCounts, readPairs, cats);
 
