@@ -40,13 +40,12 @@ Copyright 2007, 2008 Daniel Zerbino (zerbino@ebi.ac.uk)
 typedef struct miniConnection_st MiniConnection;
 
 struct miniConnection_st {
-	Coordinate distance;
-	double variance;
 	Connection *frontReference;
 	Connection *backReference;
 	NodeList *nodeList;
-};
-
+	double variance;
+	IDnum distance;
+}  ATTRIBUTE_PACKED;
 
 // Global pointers
 static Graph *graph;
@@ -85,7 +84,7 @@ static NodeList *recordNode(Node * node)
 
 static void destroyNodeList(NodeList * nodeList)
 {
-	//printf("Destroy NL  %p > %p > %p\n", nodeList->previous, nodeList, nodeList->next);
+	//velvetLog("Destroy NL  %p > %p > %p\n", nodeList->previous, nodeList, nodeList->next);
 
 	if (nodeList->previous != NULL)
 		nodeList->previous->next = nodeList->next;
@@ -235,7 +234,7 @@ static void integrateDerivativeDistances(Connection * connect,
 	if (!getUniqueness(reference))
 		return;
 
-	//printf("Opposite node %li length %li at %li ± %f\n", getNodeID(reference), getNodeLength(reference), getConnectionDistance(connect), getConnectionVariance(connect));
+	//velvetLog("Opposite node %li length %li at %li ± %f\n", getNodeID(reference), getNodeLength(reference), getConnectionDistance(connect), getConnectionVariance(connect));
 
 	baseDistance = getConnectionDistance(connect);
 	baseVariance = getConnectionVariance(connect);
@@ -265,7 +264,7 @@ static void integrateDerivativeDistances(Connection * connect,
 
 		// Avoid over-projection
 		if (distance < min_distance) {
-			//printf("Node %li not at distance %li± %f (min %li)\n", destinationID, distance, variance, min_distance);
+			//velvetLog("Node %li not at distance %li± %f (min %li)\n", destinationID, distance, variance, min_distance);
 			continue;
 		}
 
@@ -280,10 +279,10 @@ static void integrateDerivativeDistances(Connection * connect,
 					    distance, variance, NULL, NULL,
 					    true);
 
-		//printf("Node %li now at distance %li\n", destinationID, localConnect->distance);
+		//velvetLog("Node %li now at distance %li\n", destinationID, localConnect->distance);
 	}
 
-	//printf("%li secondary distances added\n", counter);
+	//velvetLog("%li secondary distances added\n", counter);
 }
 
 static void markInterestingNodes(Node * node)
@@ -547,7 +546,7 @@ static void recenterNode(Node * node, Coordinate oldLength)
 	    getNodeLength(node) / 2 - BACKTRACK_CUTOFF;
 	MiniConnection *localConnect;
 
-	//puts("Recentering node");
+	//velvetLog("Recentering node\n");
 
 	for (connect = getConnection(node); connect != NULL;
 	     connect = next) {
@@ -555,7 +554,7 @@ static void recenterNode(Node * node, Coordinate oldLength)
 		incrementConnectionDistance(connect, -distance_shift);
 
 		if (getConnectionDistance(connect) < min_distance) {
-			//printf("Unrecording %li\n",
+			//velvetLog("Unrecording %li\n",
 			//       -getNodeID(getConnectionDestination(connect)));
 			localConnect =
 			    &localScaffold[-getNodeID(getConnectionDestination(connect))
@@ -637,19 +636,19 @@ static void adjustShortReads(Node * target, Node * source)
 
 static void adjustLongReads(Node * target, Node * source)
 {
-	PassageMarker *marker;
+	PassageMarkerI marker;
 	Coordinate nodeLength = getNodeLength(source);
 
-	for (marker = getMarker(source); marker != NULL;
+	for (marker = getMarker(source); marker != NULL_IDX;
 	     marker = getNextInNode(marker))
 		incrementFinishOffset(marker, nodeLength);
 }
 
-static boolean goesToNode(PassageMarker * marker, Node * node)
+static boolean goesToNode(PassageMarkerI marker, Node * node)
 {
-	PassageMarker *current;
+	PassageMarkerI current;
 
-	for (current = marker; current != NULL;
+	for (current = marker; current != NULL_IDX;
 	     current = getNextInSequence(current))
 		if (getNode(current) == node)
 			return true;
@@ -657,12 +656,12 @@ static boolean goesToNode(PassageMarker * marker, Node * node)
 	return false;
 }
 
-static boolean comesFromNode(PassageMarker * marker, Node * node)
+static boolean comesFromNode(PassageMarkerI marker, Node * node)
 {
 	Node *target = getTwinNode(node);
-	PassageMarker *current;
+	PassageMarkerI current;
 
-	for (current = getTwinMarker(marker); current != NULL;
+	for (current = getTwinMarker(marker); current != NULL_IDX;
 	     current = getNextInSequence(current))
 		if (getNode(current) == target)
 			return true;
@@ -670,12 +669,12 @@ static boolean comesFromNode(PassageMarker * marker, Node * node)
 	return false;
 }
 
-static void reconnectPassageMarker(PassageMarker * marker, Node * node,
-				   PassageMarker ** ptr)
+static void reconnectPassageMarker(PassageMarkerI marker, Node * node,
+				   PassageMarkerI * ptr)
 {
-	PassageMarker *current;
-	PassageMarker *next = getNextInSequence(marker);
-	PassageMarker *tmpMarker;
+	PassageMarkerI current;
+	PassageMarkerI next = getNextInSequence(marker);
+	PassageMarkerI tmpMarker;
 
 	for (current = marker; getNode(current) != node;
 	     current = getPreviousInSequence(current));
@@ -687,8 +686,8 @@ static void reconnectPassageMarker(PassageMarker * marker, Node * node,
 		tmpMarker = getPreviousInSequence(marker);
 		if (*ptr == marker || *ptr == getTwinMarker(marker))
 			*ptr = getNextInNode(*ptr);
-		setNextInSequence(marker, NULL);
-		setPreviousInSequence(NULL, marker);
+		setNextInSequence(marker, NULL_IDX);
+		setPreviousInSequence(NULL_IDX, marker);
 		destroyPassageMarker(marker);
 	}
 }
@@ -696,10 +695,10 @@ static void reconnectPassageMarker(PassageMarker * marker, Node * node,
 static void concatenateLongReads(Node * node, Node * candidate,
 				 Graph * graph)
 {
-	PassageMarker *marker, *tmpMarker;
+	PassageMarkerI marker, tmpMarker;
 
 	// Passage marker management in node:
-	for (marker = getMarker(node); marker != NULL;
+	for (marker = getMarker(node); marker != NULL_IDX;
 	     marker = getNextInNode(marker)) {
 		if (!goesToNode(marker, candidate))
 			incrementFinishOffset(marker,
@@ -707,7 +706,7 @@ static void concatenateLongReads(Node * node, Node * candidate,
 	}
 
 	// Swapping new born passageMarkers from candidate to node
-	for (marker = getMarker(candidate); marker != NULL;
+	for (marker = getMarker(candidate); marker != NULL_IDX;
 	     marker = tmpMarker) {
 		tmpMarker = getNextInNode(marker);
 
@@ -1192,7 +1191,7 @@ static boolean expandLongNodes(boolean force_jumps)
 
 static void cleanMemory()
 {
-	puts("Cleaning memory");
+	velvetLog("Cleaning memory\n");
 
 	cleanScaffoldMemory();
 
@@ -1212,7 +1211,7 @@ void exploitShortReadPairs(Graph * argGraph, ReadSet * reads,
 	if (!readStartsAreActivated(graph))
 		return;
 
-	puts("Starting pebble resolution...");
+	velvetLog("Starting pebble resolution...\n");
 
 	// Prepare graph
 	resetNodeStatus(graph);
@@ -1235,5 +1234,5 @@ void exploitShortReadPairs(Graph * argGraph, ReadSet * reads,
 
 	sortGapMarkers(graph);
 
-	puts("Pebble done.");
+	velvetLog("Pebble done.\n");
 }
