@@ -52,14 +52,14 @@ SplayTable *newSplayTable(int WORDLENGTH, boolean double_strand)
 
 void destroySplayTable(SplayTable * splayTable)
 {
-	puts("Destroying splay table");
+	velvetLog("Destroying splay table\n");
 
 	destroyAllSplayTrees();
 	free(splayTable->table);
 	destroyKmerOccurenceTable(splayTable->kmerOccurenceTable);
 	free(splayTable);
 
-	puts("Splay table destroyed");
+	velvetLog("Splay table destroyed\n");
 }
 
 static int hash_kmer(Kmer * kmer)
@@ -212,7 +212,12 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 	table->lastIndex++;
 
 	currentIndex = table->lastIndex;
-	fprintf(file, "ROADMAP %d\n", currentIndex);
+	velvetFprintf(file, "ROADMAP %li\n", (long) currentIndex);
+
+	// Neglect any string shorter than WORDLENGTH :
+	if (getLength(tString) < table->WORDLENGTH) {
+		return;
+	}
 
 	// Fill in the initial word : 
 	for (readNucleotideIndex = 0;
@@ -297,7 +302,7 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 		if (!found) {
 			writeNucleotideIndex++;
 			if (!annotationClosed) {
-				fprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+				velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
 					(long) referenceSequenceID, (long long) position,
 					(long long) start, (long long) finish);
 			}
@@ -332,7 +337,7 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 			}
 			// Previous non corresponding annotation
 			else {
-				fprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+				velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
 					(long) referenceSequenceID, (long long) position,
 					(long long) start, (long long) finish);
 
@@ -351,7 +356,7 @@ static void printAnnotations(IDnum *sequenceIDs, Coordinate * coords, TightStrin
 	}
 
 	if (!annotationClosed) {
-		fprintf(file, "%ld\t%lld\t%lld\t%lld\n",
+		velvetFprintf(file, "%ld\t%lld\t%lld\t%lld\n",
 			(long) referenceSequenceID, (long long) position,
 			(long long) start, (long long) finish);
 	}
@@ -363,6 +368,7 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 	Coordinate readNucleotideIndex = 0;
 	Kmer word;
 	Kmer antiWord;
+	Kmer polyA;
 	Nucleotide nucleotide;
 	KmerOccurence * hit;
 	char line[MAXLINE];
@@ -375,6 +381,8 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 	long long longlong_var;
 	int penalty;
 
+	clearKmer(&polyA);
+
 	// Parse file for mapping info
 	while (seqFile && fgets(line, MAXLINE, seqFile)) {
 		if (line[0] == '>')
@@ -409,6 +417,17 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 #endif
 	}
 
+	// Kill silly poly-T beginnings
+	while (readNucleotideIndex < getLength(tString) && (compareKmers(&antiWord, &polyA) == 0 || compareKmers(&word, &polyA) == 0)) {
+		nucleotide = getNucleotide(readNucleotideIndex++, tString);
+		pushNucleotide(&word, nucleotide);
+#ifdef COLOR
+		reversePushNucleotide(&antiWord, nucleotide);
+#else
+		reversePushNucleotide(&antiWord, 3 - nucleotide);
+#endif
+	}
+
 	while (readNucleotideIndex < getLength(tString)) {
 		// Shift word:
 		nucleotide = getNucleotide(readNucleotideIndex, tString);
@@ -462,7 +481,7 @@ static void computeClearHSPs(TightString * tString, FILE * seqFile, boolean seco
 			coords[readNucleotideIndex] = -1;
 			readNucleotideIndex++;
 
-			for (penalty = 0; penalty < table->WORDLENGTH  - 1 && readNucleotideIndex < getLength(tString); penalty++) {
+			for (penalty = 0; readNucleotideIndex < getLength(tString) && (penalty < table->WORDLENGTH  - 1 || compareKmers(&word, &polyA) == 0 || compareKmers(&antiWord, &polyA) == 0); penalty++) {
 				nucleotide = getNucleotide(readNucleotideIndex, tString);
 				pushNucleotide(&word, nucleotide);
 
@@ -491,12 +510,6 @@ void inputSequenceIntoSplayTable(TightString * tString,
 	IDnum * sequenceIDs = NULL;
 	Coordinate * coords = NULL;
 
-	// Neglect any string shorter than WORDLENGTH :
-	if (length < table->WORDLENGTH) {
-		destroyTightString(tString);
-		return;
-	}
-
 	// If appropriate, get the HSPs on reference sequences
 	if (table->kmerOccurenceTable) {
 		length = getLength(tString);
@@ -513,7 +526,6 @@ void inputSequenceIntoSplayTable(TightString * tString,
 		free(sequenceIDs);
 		free(coords);
 	}
-	destroyTightString(tString);
 }
 
 void inputReferenceIntoSplayTable(TightString * tString,
@@ -532,11 +544,10 @@ void inputReferenceIntoSplayTable(TightString * tString,
 	table->lastIndex++;
 
 	currentIndex = table->lastIndex;
-	fprintf(file, "ROADMAP %d\n", currentIndex);
+	velvetFprintf(file, "ROADMAP %li\n", (long) currentIndex);
 
 	// Neglect any string shorter than WORDLENGTH :
 	if (getLength(tString) < table->WORDLENGTH) {
-		destroyTightString(tString);
 		return;
 	}
 
@@ -585,7 +596,6 @@ void inputReferenceIntoSplayTable(TightString * tString,
 		kmerIndex++;
 	}
 
-	destroyTightString(tString);
 	return;
 }
 
@@ -595,8 +605,11 @@ static Coordinate countReferenceKmers(ReadSet * reads, int wordLength) {
 	
 
 	for (readIndex = 0; readIndex < reads->readCount && reads->categories[readIndex] == REFERENCE; readIndex++)	
-		if (getLength(reads->tSequences[readIndex]) >= wordLength)
-			length += getLength(reads->tSequences[readIndex]) - wordLength + 1;	
+	{
+		Coordinate tmpLength = getLength(getTightStringInArray(reads->tSequences, readIndex));
+		if (tmpLength >= wordLength)
+			length += tmpLength - wordLength + 1;
+	}
 
 	return length;
 }
@@ -607,7 +620,7 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 {
 	IDnum index;
 	IDnum sequenceCount = reads->readCount;
-	TightString **array;
+	TightString *array;
 	FILE *outfile = fopen(filename, "w");
 	FILE *seqFile = NULL;
 	IDnum kmerCount;
@@ -618,13 +631,13 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 	if (outfile == NULL)
 		exitErrorf(EXIT_FAILURE, true, "Couldn't write to file %s", filename);
 	else
-		printf("Writing into roadmap file %s...\n", filename);
+		velvetLog("Writing into roadmap file %s...\n", filename);
 
 	// Count reference sequences
 	for (index = 0; index < reads->readCount && reads->categories[index] == REFERENCE; index++)
 		referenceSequenceCount++;
 
-	fprintf(outfile, "%ld\t%ld\t%i\t%hi\n", (long) sequenceCount, (long) referenceSequenceCount, table->WORDLENGTH, (short) table->double_strand);
+	velvetFprintf(outfile, "%ld\t%ld\t%i\t%hi\n", (long) sequenceCount, (long) referenceSequenceCount, table->WORDLENGTH, (short) table->double_strand);
 
 	if (reads->tSequences == NULL)
 		convertSequences(reads);
@@ -637,7 +650,7 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 		if (seqFile == NULL)
 			exitErrorf(EXIT_FAILURE, true, "Couldn't write to file %s", seqFilename);
 		else
-			printf("Reading mapping info from file %s\n", seqFilename);
+			velvetLog("Reading mapping info from file %s\n", seqFilename);
 
 		for (index = 0; index < referenceSequenceCount + 1; index++) 
 			while (fgets(line, MAXLINE, seqFile))
@@ -645,13 +658,13 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 					break;
 	}
 
-	puts("Inputting sequences...");
+	velvetLog("Inputting sequences...\n");
 	array = reads->tSequences;
 	for (index = 0; index < sequenceCount; index++) {
 		// Prorgess report on screen
 		if (index % 100000 == 0) {
-			printf("Inputting sequence %d / %d\n", index,
-			       sequenceCount);
+			velvetLog("Inputting sequence %li / %li\n", (long) index,
+			       (long) sequenceCount);
 			fflush(stdout);
 		}
 
@@ -664,18 +677,21 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 		else if (index > 0
 		    && reads->categories[index - 1] != REFERENCE
 		    && reads->categories[index] == REFERENCE) {
-			puts("Reference sequence placed after a non-reference read!");
-			puts(">> Please re-order the filenames in your command line so as to have the reference sequence files before all the others");
+			velvetLog("Reference sequence placed after a non-reference read!\n");
+			velvetLog(">> Please re-order the filenames in your command line so as to have the reference sequence files before all the others\n");
+#ifdef DEBUG 
+			abort();
+#endif 
 			exit(0);
 		}
 
 		// Hashing the reads
 		if (reads->categories[index] == REFERENCE)
 			// Reference reads
-			inputReferenceIntoSplayTable(array[index], table, outfile);
+			inputReferenceIntoSplayTable(getTightStringInArray(array, index), table, outfile);
 		else
 			// Normal reads
-			inputSequenceIntoSplayTable(array[index], table, outfile, seqFile, second_in_pair);
+			inputSequenceIntoSplayTable(getTightStringInArray(array, index), table, outfile, seqFile, second_in_pair);
 
 		if (reads->categories[index] % 2) 
 			second_in_pair = (second_in_pair? false : true);
@@ -691,5 +707,5 @@ void inputSequenceArrayIntoSplayTableAndArchive(ReadSet * reads,
 	free(reads->tSequences);
 	reads->tSequences = NULL;
 	destroyReadSet(reads);
-	puts("Done inputting sequences");
+	velvetLog("Done inputting sequences\n");
 }
